#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
import re
import datetime
from tempfile import mkdtemp
from shutil import rmtree
from subprocess import Popen, PIPE
from PIL import Image
from woost.models.file import File
from woost.models.rendering.renderer import Renderer


class VideoFileRenderer(Renderer):
    """A content renderer that handles video files."""

    instantiable = True

    try:
        p = Popen(["which", "ffmpeg"], stdout=PIPE)
        ffmpeg_path = p.communicate()[0].replace("\n", "") or None
    except:
        ffmpeg_path = None
    try:
        p = Popen(["which", "grep"], stdout=PIPE)
        grep_path = p.communicate()[0].replace("\n", "") or None
    except:
        grep_path = None
    try:
        p = Popen(["which", "cut"], stdout=PIPE)
        cut_path = p.communicate()[0].replace("\n", "") or None
    except:
        cut_path = None
    try:
        p = Popen(["which", "sed"], stdout=PIPE)
        sed_path = p.communicate()[0].replace("\n", "") or None
    except:
        sed_path = None

    def _secs2time(self, s):
        ms = int((s - int(s)) * 1000000)
        s = int(s)
        # Get rid of this line if s will never exceed 86400
        while s >= 24*60*60: s -= 24*60*60
        h = s / (60*60)
        s -= h*60*60
        m = s / 60
        s -= m*60
        return datetime.time(h, m, s, ms)

    def _time2secs(self, d):
        return d.hour*60*60 + d.minute*60 + d.second + \
            (float(d.microsecond) / 1000000)

    def can_render(self, item, **parameters):
        return (
            self.ffmpeg_path
            and self.grep_path
            and self.cut_path
            and self.sed_path
            and isinstance(item, File)
            and item.resource_type == "video"
        )

    def render(self, item, position = None, **parameters):

        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command1 = "%s -i %s" % (self.ffmpeg_path, item.file_path)
            command2 = "%s Duration | %s -d ' ' -f 4 | %s 's/,//'" % (
                self.grep_path,
                self.cut_path,
                self.sed_path
            )
            p1 = Popen(command1, shell=True, stderr=PIPE)
            p2 = Popen(command2, shell=True, stdin=p1.stderr, stdout=PIPE)
            duration = p2.communicate()[0]

            duration_list = re.split("[.:]", duration)
            video_length = datetime.time(
                int(duration_list[0]), int(duration_list[1]),
                int(duration_list[2]), int(duration_list[3])
            )

            if position:
                time = self._secs2time(position)
            else:
                seconds = self._time2secs(video_length)
                time = self._secs2time(seconds / 2)

            if time > video_length:
                raise ValueError(
                    "Must specify a smaller position than the video duration."
                )

            command = u"%s -y -ss %s -i %s -vframes 1 -an -vcodec png -f rawvideo %s " % (
                self.ffmpeg_path,
                time.strftime("%H:%M:%S"),
                item.file_path,
                temp_image_file
            )

            p = Popen(command.split(), stdout=PIPE)
            p.communicate()

            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# Configuration
#------------------------------------------------------------------------------
translations.define("Configuration.media.audio",
    ca = u"Audio",
    es = u"Audio",
    en = u"Audio"
)

translations.define("Configuration.audio_decoders",
    ca = u"Decodificadors d'àudio",
    es = u"Decodificadores de audio",
    en = u"Audio decoders"
)

translations.define("Configuration.audio_encoders",
    ca = u"Codificadors d'àudio",
    es = u"Codificadores de audio",
    en = u"Audio encoders"
)

# AudioDecoder
#------------------------------------------------------------------------------
translations.define("AudioDecoder",
    ca = u"Decodificador d'àudio",
    es = u"Decodificador de audio",
    en = u"Audio decoder"
)

translations.define("AudioDecoder-plural",
    ca = u"Decodificadors d'àudio",
    es = u"Decodificadores de audio",
    en = u"Audio decoders"
)

translations.define("AudioDecoder.mime_type",
    ca = u"Tipus MIME",
    es = u"Tipo MIME",
    en = u"MIME type"
)

translations.define("AudioDecoder.command",
    ca = u"Comanda",
    es = u"Comando",
    en = u"Command"
)

# AudioEncoder
#------------------------------------------------------------------------------
translations.define("AudioEncoder",
    ca = u"Codificador d'àudio",
    es = u"Codificador de audio",
    en = u"Audio encoder"
)

translations.define("AudioEncoder-plural",
    ca = u"Codificadors d'àudio",
    es = u"Codificadores de audio",
    en = u"Audio encoders"
)

translations.define("AudioEncoder.identifier",
    ca = u"Identificador",
    es = u"Identificador",
    en = u"Identifier"
)

translations.define("AudioEncoder.mime_type",
    ca = u"Tipus MIME",
    es = u"Tipo MIME",
    en = u"MIME type"
)

translations.define("AudioEncoder.extension",
    ca = u"Extensió",
    es = u"Extensión",
    en = u"Extension"
)

translations.define("AudioEncoder.command",
    ca = u"Comanda",
    es = u"Comando",
    en = u"Command"
)

translations.define("AudioEncoder.timeout",
    ca = u"Temps màxim d'execució",
    es = u"Tiempo máximo de ejecución",
    en = u"Timeout"
)

translations.define("AudioEncoder.timeout_size_factor",
    ca = u"Temps d'execució addicional segons la mida del fitxer",
    es = u"Tiempo de ejecución adicional según la mida del fichero",
    en = u"Timeout file size factor"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import templates
from woost.models import Extension


translations.define("SurveysExtension",
    ca = u"Enquestes",
    es = u"Encuestas",
    en = u"Surveys"
)

translations.define("SurveysExtension-plural",
    ca = u"Enquestes",
    es = u"Encuestas",
    en = u"Surveys"
)


class SurveysExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Permet fer enquestes als visitants de la web.""",
            "ca"
        )
        self.set("description",
            u"""Permite hacer encuestas a los visitantes de la web.""",
            "es"
        )
        self.set("description",
            u"""Design and conduct surveys for the visitors of the website.""",
            "en"
        )

    def _load(self):
        from woost.extensions.surveys import (
            strings,
            survey,
            surveyblock,
            surveyresultsaction
        )

        from woost.controllers.backoffice.itemcontroller import ItemController
        from woost.extensions.surveys.surveyresultscontroller \
            import SurveyResultsController

        ItemController.survey_results = SurveyResultsController




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.models import File, VideoPlayerSettings
from woost.extensions.vimeo.vimeovideo import VimeoVideo


VideoPlayerSettings.members_order = VideoPlayerSettings.members_order + [
        "vimeo_loop",
        "vimeo_allow_fullscreen",
        "vimeo_title",
        "vimeo_byline",
        "vimeo_portrait",
        "vimeo_color"
]

VideoPlayerSettings.add_member(
    schema.Boolean(
        "vimeo_loop",
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "vimeo"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "vimeo_allow_fullscreen",
        default = True,
        listed_by_default = False,
        member_group = "vimeo"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "vimeo_title",
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "vimeo"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "vimeo_byline",
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "vimeo"
    )
)

VideoPlayerSettings.add_member(
    schema.Boolean(
        "vimeo_portrait",
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "vimeo"
    )
)

VideoPlayerSettings.add_member(
    schema.Color(
        "vimeo_color",
        listed_by_default = False,
        member_group = "vimeo"
    )
)

def player_initialization(player, settings, video):
    player.video_id = video.video_id
    player.width = settings.width
    player.height = settings.height
    player.vimeo_autoplay = settings.autoplay
    player.allow_fullscreen = settings.vimeo_allow_fullscreen
    player.vimeo_loop = settings.vimeo_loop
    player.vimeo_title = settings.vimeo_title
    player.vimeo_byline = settings.vimeo_byline
    player.vimeo_portrait = settings.vimeo_portrait
    player.vimeo_color = settings.vimeo_color

VideoPlayerSettings.player_initialization[VimeoVideo] = player_initialization




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
from cocktail import schema
from woost.models import Permission
from woost.models.messagestyles import permission_doesnt_match_style
from woost.extensions.mailer.mailinglist import MailingList


class SendEmailPermission(Permission):
    """Permission to send an email."""

    instantiable = True

    lists = schema.Collection(
        items = schema.Reference(
            type = MailingList,
        ),
        related_end = schema.Collection(),
        edit_control = "cocktail.html.CheckList"
    )

    def match(self, user, mailingList = None, verbose = False):

        if mailingList and self.lists and mailingList not in self.lists:

            if verbose:
                print permission_doesnt_match_style("Mailing list doesn't match")

            return False

        return Permission.match(self, user, verbose)




#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""A convenience script that can be launched through ipython or vanilla python
with the -i flag, to have quick access to the project's data.
"""
from woost.scripts.shell import *
from _PROJECT_MODULE_.models import *
setup_shell(locals())




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from .document import Document
from .template import Template
from .slot import Slot


class Page(Document):

    default_template = schema.DynamicDefault(
        lambda: Template.get_instance(qname = "woost.standard_template")
    )

    blocks = Slot()

